#! python3  # noqa: E265

"""
    Authentication dialog logic.
"""

# standard
from pathlib import Path

from qgis.core import QgsApplication
from qgis.PyQt import uic
from qgis.PyQt.QtWidgets import QDialog

from oslandia.gitlab_api.custom_exceptions import UnavailableUserException

# PyQGIS
from oslandia.gitlab_api.user import UserRequestManager

# Plugin
from oslandia.toolbelt import PlgLogger, PlgOptionsManager


class AuthenticationDialog(QDialog):
    def __init__(self, parent=None):
        """Dialog to define current connection as authentication config.

        :param parent: parent widget, defaults to None
        :type parent: QObject, optional
        """
        # init module and ui
        super().__init__(parent)
        uic.loadUi(Path(__file__).parent / "{}.ui".format(Path(__file__).stem), self)

        # toolbelt
        self.log = PlgLogger().log
        self.plg_settings_mngr = PlgOptionsManager()
        self.plg_settings = self.plg_settings_mngr.get_plg_settings()

        # widgets connect
        self.btn_log_in.setEnabled(False)
        self.btn_log_in.clicked.connect(self.connect)

        self.lne_client_id.textChanged.connect(self._client_config_updated)
        self.lne_client_secret.textChanged.connect(self._client_config_updated)

        self.txt_help.setHtml(self._html_documentation())

    def _html_documentation(self) -> str:
        image_path = Path(__file__).parent / ".." / "resources" / "images"
        return f"""
        <!DOCTYPE html>
        <html lang="en">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <title>GitLab Documentation</title>
        </head>
        <body>
            <p><b>If you don't have this information, you can create a new application from your GitLab account.</b></p>

            <ol>
                <li>Go to <a href="https://git.oslandia.net/-/profile/applications" target="_blank">https://git.oslandia.net/-/profile/applications</a></li>
                <li>Define a name for your application (for example, <strong>Oslandia QGis plugin</strong>).</li>
                <li>Set the Redirect URI to <strong>http://127.0.0.1:7070/callback</strong></li>
                <li>Enable the scopes <strong>api</strong>, <strong>read_api</strong>, and <strong>read_user</strong>.
                    <br>
                    <img src="{image_path}/authentification/gitlab_conf.png" alt="GitLab configuration page">
                </li>
                <li>Save the application. A new page will be displayed.
                    <br>
                    <img src="{image_path}/authentification/gitlab_token.png" alt="GitLab configuration page">
                </li>
                <li>Copy the <strong>Application ID</strong> into the corresponding field.</li>
                <li>Copy the <strong>Secret</strong> into the corresponding field.</li>
                <li>Log in, a page should launch in your browser to authorize the application.
                    <br>
                    <img src="{image_path}/authentification/gitlab_access_authenticated.png" alt="GitLab configuration page">
                </li>
                <li>Authorize the application.</li>
            </ol>

            <p><b>You can now connect to your Oslandia projects!</b></p>
        </body>
        </html>
        """

    def _client_config_updated(self) -> None:
        """Enable connection if client id and client secret are defined"""
        self.btn_log_in.setEnabled(
            self.lne_client_id.text().strip() != ""
            and self.lne_client_secret.text().strip() != ""
        )

    def connect(self) -> None:
        """Check connection parameter and define current geotuileur authentication config
        if connection is valid.
        """
        # Retrieve form data
        client_id = self.lne_client_id.text()
        client_secret = self.lne_client_secret.text()

        # Create new authentication config and add it to manager
        auth_manager = QgsApplication.authManager()
        new_auth = self.plg_settings.create_auth_config(client_id, client_secret)
        auth_manager.storeAuthenticationConfig(new_auth, True)
        self.log(
            message="Credentials saved into QGIS Authentication Manager as "
            f" {new_auth.name()} ({new_auth.id()})",
            log_level=4,
        )

        # If connection valid, remove previous config and use new created config,
        # otherwise remove created config
        if self.check_connection(new_auth.id()):
            if self.plg_settings.authentification_config_id:
                auth_manager.removeAuthenticationConfig(
                    self.plg_settings.authentification_config_id
                )
            self.plg_settings.authentification_config_id = new_auth.id()
            self.plg_settings_mngr.save_from_object(self.plg_settings)

            # Validate dialog
            self.accept()
        else:
            auth_manager.removeAuthenticationConfig(new_auth.id())

    def check_connection(self, qgis_auth_id: str) -> bool:
        """Check if connection is valid for a qgis authentication id.
        Display a message box with user name and last name if connection valid, or
        message with error message otherwise.

        :param qgis_auth_id: qgis authentication id to use
        :type qgis_auth_id: str

        :return: True if connection is valid for qgis_auth_id, False otherwise
        :rtype: bool
        """

        res = True

        # Check connection to API by getting token user information
        try:
            manager = UserRequestManager()
            manager.plg_settings.authentification_config_id = qgis_auth_id
            user = manager.get_current_user()

            self.log(
                message=self.tr(f"Welcome {user.name}!"),
                log_level=3,
                push=True,
                duration=5,
            )

        except UnavailableUserException as exc:
            self.log(
                message=self.tr(f"Invalid connection parameters: {exc}"),
                log_level=2,
                push=True,
                duration=30,
            )
            res = False

        return res
